/*

So, a bunch of my boost::regex code started crashing when I upgraded to msvc9.

After some frustration, I discovered that I could fix the crashes by defining BOOST_NO_STD_ALLOCATOR --
a flag currently defined for msvc8, but not msvc9.

I'm fairly sure that these crashes are due to some subtle bug in the microsoft compiler -- though
how said bug is related to the std allocator issue is unclear to me. 

I've had trouble isolating the cause of the bug -- as I try to simplify the code to figure out 
exactly what it is that's causing the crash, surpringly subtle changes can cause the bug to disapear.

All that said -- this is currently simplest test code I've got that replicates the crash.

I'm running it against the boostpro packaged windows installers for 1.38 -- I'm afraid I'm
too pressed for time atm to compile the latest sources myself.  
If this issue is fixed in 1.39 or the svn, I apologize.

The bug only occures when the project is compiled in release mode.

*/


// undefining this reliably fixes the bug.
#define _SECURE_SCL 0

// defining this reliably fixes the bug.
//#define BOOST_NO_STD_ALLOCATOR

#include <stdio.h>
#include <iostream>
#include <boost/regex.hpp>
#include <string>




int main() {


	using namespace std;
	using namespace boost;

	string s="foo bar";

	regex token=regex("(foo)|(bar)");
	regex seperator=regex("\\s+");

	const string &data=s;

	boost::match_results<std::string::const_iterator> what;
	
	std::string::const_iterator start=data.begin();


	// bizzarly, uncommenting this line sometimes appears to fix the bug.
	//start=data.begin();

	regex_search(start,data.end(),what,token);

	start=what[0].second;

	// this line runs fine, so, the iterator appears to be valid...
	cout << string(start,data.end()) << endl;


	// this is the function call that causes the crash.
	// (msvc's debugger is showing me an 
	// "Access violation reading location" error.)
	printf("(pre crash)\n");
	regex_search(start,data.end(),what,seperator);
	printf("(post crash)\n");

	return 0;
}